<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Templates;
use App\Exception;

class MenuController {
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Service\MenuService $menuService,
                                 \App\Service\PriceService $priceService,
                                 \App\Util\Util $util) {

        $this->connection = $connection;
        $this->menuService = $menuService;
        $this->priceService = $priceService;
        $this->util        = $util;
    }
    
    public function welcome(Request $request, Response $response) {
        global $CLIENT_DATA;
        
        $config      = $CLIENT_DATA;
        $sql         = "SELECT CDFILIAL FROM FILIAL";
        $result      = $this->connection->fetchAll($sql);
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'client'       => $config['client'],
            'version'      => $versionInfo['version'],
            'revision'     => $versionInfo['revision'],
            'date'         => $versionInfo['date'],
            'filialCount'  => count($result)
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }
    
    public function loadMenu(Request $request, Response $response) {
        ini_set('memory_limit', '-1');
        try {
            
			$row = $request->getParsedBody();
			
            if(empty($row)) 
                throw new \Exception("Empty body request.");

			$CDFILIAL = $row["branchId"];
			
			$data     = $this->menuService->loadMenu($CDFILIAL);
        
            return $response->withJson([
                    'success' => true, 
                    'menu' => $data
            ]);

		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getBranchs(Request $request, Response $response) {
        try {

			$data = $this->menuService->getBranchs();

            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getMenus(Request $request, Response $response) {
        try {
			$data = $this->menuService->getMenus();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getProducts(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            
            $cdFilial   = $params['branchId'];
            $nrConfTela = $params['menuId'];
            
			$data = $this->menuService->getProducts($cdFilial, $nrConfTela);
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getMenusUpdates(Request $request, Response $response) {
        try {
			$data = $this->menuService->getMenusUpdates();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getPricingTables(Request $request, Response $response) {
        try {
			$data = $this->menuService->getPricingTables();
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    
    public function getProductsByPricingTable(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            
            $cdTabePrec = $params['tableId'];
            
			$data = $this->menuService->getProductsByPricingTable($cdTabePrec);
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }
    

    //=======================================================
    public function getRestaurants(Request $request, Response $response) {
        try {
            $branchs = $this->menuService->getBranchs();
            if(empty($branchs))
                throw new \Exception ("Não foi encontrado nenhuma Unidade configurada para venda. Consulte o manual de cadastros!");
            
            
            $return = [];
            foreach($branchs as $branch){
    			$data = [
    			    
                    "id"             => $branch['branchId'], 
                    "restaurantName" => $branch['branchName'], 
                    "description"    => $branch['branchDescription'], 
                    "tags"           => $branch['branchTags'], 
                    "headerImage"    => $branch['branchHeaderImage']

                ];
                
                array_push($return, $data);
            }
 
            return $response->withJson($return);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getRestaurantsDetails(Request $request, Response $response) {
        try {
            
			$params = $request->getParsedBody();
			
            if(empty($params)) 
                throw new \Exception("Empty body request.");

			$id = $params["id"];
            $branchDetails = $this->menuService->getBranchData($id);

            if(empty($branchDetails))
                throw new \Exception ("Unidade {$id} não encontrada.");
            
            $storeId = $branchDetails['CDLOJA'];
            $cashierId = $branchDetails['CDCAIXA'];
            $payments = $this->menuService->getPayments($branchDetails['CDFILIAL'], $cashierId);
            $address = "{$branchDetails['DSENDEFILISEP']}, {$branchDetails['NRENDEFILISEP']}, {$branchDetails['NMBAIRFILI']}, {$branchDetails['NMMUNICIPIO']} - {$branchDetails['SGESTADO']}";
            $openingHours = $this->menuService->getOpeningHours($branchDetails['CDFILIAL'], $storeId);;
            
			$data = [
                "id"                    => $branchDetails['CDFILIAL'], 
                "restaurantDescription" => '', 
                "restaurantName"        => $branchDetails['NMFILIAL'], 
                "restaurantImg"         => [],
                "chefImage"             => '', 
                "tags"                  => [], 
                "restaurantInfo"        => [
                    "email"             => $branchDetails['DSEMAILFILI'], 
                    "paymentMethods"    => $payments, 
                    "hours"             => isset($openingHours) ? $openingHours : '', 
                    "address"           => [
                        "location"      => !empty($address) ? ucwords(strtolower($address)) : '', 
                        "link"          => "" 
                    ] 
                ] 
            ]; 
 
            return $response->withJson($data);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }

    public function getMenu(Request $request, Response $response) {
        try {
            
			$params = $request->getParsedBody();
			
            if(empty($params)) 
                throw new \Exception("Empty body request.");

			$id = $params["id"];
			
			$branchDetails = $this->menuService->getBranchDetails($id);
			if(empty($branchDetails))
			    throw new \Exception ("Unidade {$id} não encontrada.");
			
			$branchId = $branchDetails['CDFILIAL'];
			$storeId = $branchDetails['CDLOJA'];
			$cashierId = $branchDetails['CDCAIXA'];
			$standardCustomer = $branchDetails['CDCLIENTE'];
			
			$confTela = $this->menuService->getConftelaCaixa($branchId, $cashierId);
            $priceTable = $this->priceService->getPriceTable($standardCustomer, $branchId, $storeId);
            $menu =  $this->menuService->BuildMenu($confTela, $priceTable);
            
            return $response->withJson($menu);
		} catch (\Exception $e) {
			return $response->withStatus(400)->withJson([
                'success'      => false, 
                'errorMessage' => $e->getMessage()
            ]);
		}
    }


        
}